@@ -12,6 +12,7 @@ import platform
 import inspect
 import warnings
 from io import StringIO, BytesIO
+from time import time
 
 import sys
 if sys.version_info[0] < 3:
@@ -273,10 +274,10 @@ class Connection(object):
         err_default = "Unable to preview protocol"
         return self.post(
             route,
-            json={"protocol": json.dumps(protocol)},
+            json={"protocol": protocol},
             allow_redirects=False,
             status_response={
-                '302': lambda resp: resp.headers['Location'],
+                '200': lambda resp: resp.json()["key"],
                 'default': lambda resp: RuntimeError(err_default)
             }
         )
@@ -413,6 +414,16 @@ class Connection(object):
                                launch_request_id=launch_request_id)
         return self.get(route)
 
+    def resolve_quick_launch_inputs(self, raw_inputs, project_id=None,
+                                    quick_launch_id=None):
+        """Resolves `raw_inputs` to `inputs` for quick_launch"""
+        route = self.get_route(
+            'resolve_quick_launch_inputs',
+            project_id=project_id,
+            quick_launch_id=quick_launch_id
+        )
+        return self.post(route, json=raw_inputs)
+
     def get_protocols(self):
         """Get list of available protocols"""
         route = self.get_route('get_protocols')
@@ -559,6 +570,50 @@ class Connection(object):
         route = self.get_route('dataset', data_id=data_id, key=key)
         return self.get(route)
 
+    def _get_uploads_from_key(self, key):
+        """Fetches uploads for a data upload key
+
+        Parameters
+        ----------
+        key : str
+            data upload key
+
+        Returns
+        -------
+        requests.Response
+        """
+        return self.get(
+            route=self.get_route(method="get_uploads", key=key),
+            status_response={'200': lambda resp: resp},
+            stream=True
+        )
+
+    def attachments(self, data_id):
+        """Fetches all attachments for a given dataset id
+
+        Parameters
+        ----------
+        data_id : str
+            dataset id
+
+        Returns
+        -------
+        dict(str, bytes)
+            a dict of attachment names and contents
+
+        """
+        dataset_route = self.get_route("dataset_short", data_id=data_id)
+        dataset_attachments = self.get(dataset_route).get("attachments")
+        attachment_names = [
+            os.path.basename(_.get("name", ""))
+            for _ in dataset_attachments
+        ]
+        attachment_contents = [
+            self._get_uploads_from_key(_.get("key")).content
+            for _ in dataset_attachments
+        ]
+        return dict(zip(attachment_names, attachment_contents))
+
     def datasets(self, project_id=None, run_id=None, timeout=30.0):
         """Get datasets belonging to run"""
         route = self.get_route('datasets', project_id=project_id, run_id=run_id)
@@ -608,7 +663,7 @@ class Connection(object):
         try:
             file_path = os.path.expanduser(file_path)
             file_handle = open(file_path, 'rb')
-            name = file_handle.name
+            name = os.path.basename(file_handle.name)
         except (AttributeError, FileNotFoundError) as e:
             raise ValueError("'file' has to be a valid filepath")
 
@@ -665,12 +720,12 @@ class Connection(object):
         response: dict
             JSON-formatted response
         """
-        s3_key = self.upload_to_uri(file_handle, content_type, title, name)
+        upload_id = self.upload_to_uri(file_handle, content_type, title, name)
         upload_datasets_route = self.get_route("upload_datasets")
         upload_resp = self.post(
             upload_datasets_route,
             json={
-                "s3_key": s3_key,
+                "upload_id": upload_id,
                 "file_name": name,
                 "title": title,
                 "run_id": run_id,
@@ -678,8 +733,8 @@ class Connection(object):
                 "analysis_tool_version": analysis_tool_version
             },
             status_response={
-                '404': lambda resp: "[404] Please double-check your parameters "
-                                    "and ensure they are valid."
+                '404': lambda resp: "[404] Please double-check your parameters"
+                                    " and ensure they are valid."
             }
         )
 
@@ -687,8 +742,8 @@ class Connection(object):
 
     def upload_to_uri(self, file_handle, content_type, title, name):
         """
-        Helper for uploading files via the `upload_uri` route
-        
+        Helper for uploading files via the `upload` route
+
         Parameters
         ----------
         file_handle: file_handle
@@ -705,11 +760,25 @@ class Connection(object):
         key: str
             s3 key
         """
-        uri_route = self.get_route('upload_uri')
-        uri_resp = self.post(uri_route, data=json.dumps({"name": title}))
+        # TODO:
+        # Currently, we are passing `0` for file_size as it doesn't really
+        # matter for non multipart uploads, though it would be better to
+        # supply the correct value.
+        data = {
+            "attributes": {
+                "file_name": name,
+                "file_size": 0,
+                "last_modified": int(time()),
+                "is_multipart": False
+            }
+        }
+
+        uri_route = self.get_route('upload')
+        uri_resp = self.post(uri_route, data=json.dumps({"data": data}))
+
         try:
-            key = uri_resp['key']
-            uri = uri_resp['uri']
+            upload_id = uri_resp['data']['id']
+            upload_uri = uri_resp['data']['attributes']['upload_url']
         except KeyError as e:
             raise RuntimeError("Unexpected payload returned for upload_dataset")
 
@@ -721,18 +790,18 @@ class Connection(object):
                 raise ValueError("Unable to convert read buffer to bytes")
 
         headers = {
-            "Content-Disposition": "attachment; filename='{}'".format(name),
+            "Content-Disposition": "attachment; filename={}".format(name),
             "Content-Type": content_type
         }
         headers = {k: v for k, v in headers.items() if v}
         self.put(
-            uri,
+            upload_uri,
             data=file_handle,
             custom_request=True,
             headers=headers,
             status_response={'200': lambda resp: resp}
         )
-        return key
+        return upload_id
 
     def get_zip(self, data_id, file_path=None):
         """
